#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""GroupSpace main class"""

# Python imports
import string

# CMF imports
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFPlone.PloneFolder import PloneFolder
from AccessControl import ClassSecurityInfo
from OFS import Folder

from global_symbols import *
from Globals import InitializeClass

# GroupSpace definition
factory_type_information = {
  'id'             : 'GroupSpace',
  'portal_type'    : 'GroupSpace',
  'meta_type'      : 'GroupSpace',
  'description'    : 'In GroupSpaces, members can work together on documents.',
  'content_icon'   : 'GroupSpace.gif',
  'product'        : 'GrufSpaces',
  'factory'        : 'addGroupSpace',
  'immediate_view' : 'GroupSpace_listing',
  'filter_content_types' : 0,
  'allowed_content_types': (),
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${folder_url}/GroupSpace_listing',
    'permissions'  : (CMFCorePermissions.View, ),
##    'category'      : 'folder',
    },
    {
    'id'            : 'roles',
    'name'          : 'Roles',
    'action'        : 'string:${folder_url}/GroupSpace_rolesForm',
    'permissions'   : (GroupSpace_assignRolePermission,),
##    'category'      : 'folder',
    },
    {
    'id'            : 'email',
    'name'          : 'Email',
    'action'        : 'string:${folder_url}/GroupSpace_emailForm',
    'permissions'   : (GroupSpace_sendMailPermission,),
##    'category'      : 'folder',
    },
    {
    'id'            : 'local_roles',
    'name'          : 'Sharing',
    'action'        : 'string:${folder_url}/folder_localrole_form',
    'permissions'   : (GroupSpace_assignRolePermission,),
##    'category'      : 'folder',
    },
    {
    'id'           : 'edit',
    'name'         : 'Properties',
    'action'       : 'string:${folder_url}/GroupSpace_editForm',
    'permissions'  : (CMFCorePermissions.ModifyPortalContent,),
##    'category'      : 'folder',
    },
  ),
  }

# The usual factory
def addGroupSpace(self, id, title = '', REQUEST = {}):
    """
    Factory method for a GroupSpace object
    """
    obj = GroupSpace(id, title)
    self._setObject(id, obj)
    gs = getattr(self, id)
    # Hook for adding your own services in Python Scripts
    try:
        gs.add_groupspace_services()
    except:
        pass
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')

class GroupSpace(PloneFolder):
    """GroupSpace class"""
    meta_type = "GroupSpace"
    security = ClassSecurityInfo()
    __implements__ = PloneFolder.__implements__

    #                                                                           #
    #                         GROUPSPACE-SPECIFIC METHODS                       #
    #                                                                           #

    # Security comments:
    #   GroupMembers adding / deletion is protected by the ManageProperties permission.
    #   Members listing is allowed upon AccessContentsInformation permission

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceMemberUsers")
    def listGroupSpaceMemberUsers(self,):
        """
        listGroupSpaceMemberUsers(self,) => List actual members, by inspecting each group's members
        """
        return self.listMemberIds()

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceVisitorUsers")
    def listGroupSpaceVisitorUsers(self,):
        """
        listGroupSpaceVisitorUsers(self,) => List actual visitors, by inspecting each group's visitors
        """
        return self.listVisitorIds()

    security.declareProtected(GroupSpace_sendMailPermission, "sendMail")
    def sendMail(self, mail_text):
        """
        Send mail from groupspace
        """
        
        host = self.MailHost
        host.send(mail_text)
        
        
    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listMemberIds")
    def listMemberIds(self):
        """
        listMemberIds(self,) => Return a list of member ids for this groupspace.
        This will return both users and groups.
        """
        return self.users_with_local_role("GroupMember")

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listVisitorIds")
    def listVisitorIds(self):
        """
        listVisitorIds(self,) => Return a list of visitor ids for this groupspace.
        This will return both users and groups.
        """
        return self.users_with_local_role("GroupVisitor")

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "getMainOwner")
    def getMainOwner(self,):
        """
        getMainOwner(self,) => Return the main owner of the group space.
        It's likely that this will be the person in charge of maintaining it ! :-)
        """
        return self.listGroupOwners()[0]


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupOwners")
    def listGroupOwners(self,):
        """
        listGroupOwners(self,) => Return a list of all persons having 'Owner' role in this group space.
        """
        ret = self.users_with_local_role("Owner")

        # XXX TODO:
        # XXX HAVE TO EXTEND THIS LIST TO ALL PEOPLE HAVING THE 'OWNER' ROLE,
        # XXX EVEN IF IT'S NOT A LOCAL ROLE !
        return ret

        


    #                                           #
    #           Various utilities               #
    #                                           #

    security.declarePublic("canChange")
    def canChange(self,):
        """
        canChange(self,) => return true if the current user can change things in here
        """
        mtool = getToolByName(self, 'portal_membership')
        return mtool.checkPermission(CMFCorePermissions.ManageProperties, self)

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listMembersForDisplay")
    def listMembersForDisplay(self,):
        """
        listMembersForDisplay(self,) => Same as listMemberIds() but with an accurate structure to
        display information.

        - Users are filtered
        - Roles are listed
        - Kind (user, group or unknown) is specified

        This method returns a list of tuples (massagedUsername, userType, actualUserName)
        """
        ret = []
        for usr_id in self.listMemberIds():
            obj = self.acl_users.getUser(usr_id)
            if not obj:
                # User not in the first acl_users
                ret.append((usr_id, "unknown", usr_id, ))
                continue
            if obj.isGroup():
                ret.append((obj.getUserName(), "group", usr_id, ))
            else:
                ret.append((obj.getUserName(), "user", usr_id, ))
        return ret

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "setInitialGroup")
    def setInitialGroup(self, group):
        # The group that we create the workspace for is automatically
        # becoming GroupMember
        self.manage_setLocalRoles(group.getId(), ['GroupMember'])

    #                                                                   #
    #           CollabSpace Interface                                   #
    #                                                                   #
    # Implementation of the CollabSpace Interface as defined in         #
    # PloneGroupSpace by Helmut Merz:                                   #
    #                                                                   #
    # http://cvs.sourceforge.net/viewcvs.py/collective/PloneGroupSpace/ # 
    #                                                                   #

    def getValidRoles(self):
        """ May be used to show a list of roles to select from in the user interface"""
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabRoles(self):
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabUsers(self, role=None):
        # Return all users with a local role
        if role is None:
            # Get Collab Users for all roles
            checkRoles = self.getCollabRoles()
        else:
            # Get Collab Users only for the given role
            checkRoles = [role]
        result = []
        lr = self.get_local_roles()
        for (user, roles) in lr:
            for r in roles:
                if r in checkRoles:
                    result.append(user)
                    break # One role is enough
        return tuple(result)

    def getCollabUserObjects(self, role=None):
        """ Returns real user objects - Member or Group - not just the ID.
        """
        userids = self.getCollabUsers(role)
        pg = getToolByName(self, 'portal_groups')
        pm = getToolByName(self, 'portal_membership')
        result = []
        for userid in userids:
            obj = pg.getGroupById(userid)
            if obj:
                pass
            else:
                obj = pm.getMemberById(userid)
            if obj:
                result.append(obj)
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'assignCollabRoleToUser')
    def assignCollabRoleToUser(self, role, userid):
        if role not in self.getCollabRoles():
            raise ValueError, 'Role %s is not registered as a Collab Role here.' % role
        pm = getToolByName(self, 'portal_membership')
        user = pm.getMemberById(userid)
        if not user:
            raise ValueError, 'Member %s does not exist.' % userid
        self.manage_addLocalRoles(user.getId(), [role])
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterAssignToCollabRole(role, user.getId(), user_type="user")

    security.declareProtected(GroupSpace_assignRolePermission, 'assignCollabRoleToGroup')
    def assignCollabRoleToGroup(self, role, groupid):
        if role not in self.getCollabRoles():
            raise ValueError, 'Role %s is not registered as a Collab Role here.' % role
        pg = getToolByName(self, 'portal_groups')
        group = pg.getGroupById(groupid)
        if not group:
            raise ValueError, 'Group %s does not exist.' % userid
        self.manage_addLocalRoles(group.getId(), [role])
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterAssignToCollabRole(role, group.getId(), user_type="group")

    def getCollabUserDictionary(self):
        """ A convenience function to get the full information on assigned
            users with one method call. Returns a dictionary with roles as keys and
            a tuple of assigned users as the values.
        """
        result = {}
        for role in self.getCollabRoles():
            users = self.getCollabUsers(role)
            if users:
                result[role] = users
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'removeCollabRoleFromUser')
    def removeCollabRoleFromUser(self, role, userid):
        pm = getToolByName(self, 'portal_membership')
        member = pm.getMemberById(userid)
        if not member:
            raise ValueError, 'Member %s does not exist.' % userid
        lr = []
        try:
            lr = list(self.get_local_roles_for_userid(member.getId()))
        except:
            pass
        if role in lr:
            lr.remove(role)
        if lr:
            self.manage_setLocalRoles(member.getId(), lr)
        else:
            try:
                self.manage_delLocalRoles([member.getId()])
            except:
                pass
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterRemoveFromCollabRole(role, member.getId(), user_type="group")

    security.declareProtected(GroupSpace_assignRolePermission, 'removeCollabRoleFromGroup')
    def removeCollabRoleFromGroup(self, role, groupid):
        pg = getToolByName(self, 'portal_groups')
        group = pg.getGroupById(groupid)
        if not group:
            raise ValueError, 'Group %s does not exist.' % groupid
        lr = []
        try:
            lr = list(self.get_local_roles_for_userid(group.getId()))
        except:
            pass
        if role in lr:
            lr.remove(role)
        if lr:
            self.manage_setLocalRoles(group.getId(), lr)
        else:
            try:
                self.manage_delLocalRoles([group.getId()])
            except:
                pass
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterRemoveFromCollabRole(role, group.getId(), user_type="group")

    def _afterAssignToCollabRole(self, role, user, user_type):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to register the assignment with the Member or Group object assigned.
        """
        pass

    def _afterRemoveFromCollabRole(self, role, user, user_type):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to remove the assignment from the Member or Group object assigned.
        """
        pass

    security.declareProtected(CMFCorePermissions.View, 'getCurrentGroupspace')
    def getCurrentGroupspace(self):
        """getCurrentGroupspace(self, ) => return itself
        """
        return self.restrictedTraverse('')

    #                                                                   #
    #           Extensions to the CollabSpace Interface                 #
    #                                                                   #

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "getMainCollabUserRole")
    def getMainCollabUserRole(self, user):
        # Return the main role of a user to be shown in the interface.
        roles= user.getRolesInContext(self)
        role = None
        if 'GroupLeader' in roles:
            role = 'GroupLeader'
        elif 'GroupAdmin'in roles:
            role = 'GroupAdmin'
        elif 'GroupMember' in roles:
            role = 'GroupMember'
        elif 'GroupVisitor' in roles:
            role = 'GroupVisitor'
        return role

# Intialize the GroupSpace class, setting up security.
InitializeClass(GroupSpace)

